@@ -27,7 +27,6 @@
 */
 
 #include <stdint.h>
-#include <sys/types.h>
 
 enum
 {
@@ -42,4 +41,3 @@ struct idna_table
 };
 
 extern const struct idna_table idna_table[];
-extern const size_t idna_table_size;
