@@ -1,18 +1,29 @@
 /* idna.c - implementation of high-level IDNA processing function
-   Copyright (C) 2011 Simon Josefsson
+   Copyright (C) 2011-2016 Simon Josefsson
 
-   This program is free software: you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation, either version 3 of the License, or
-   (at your option) any later version.
+   Libidn2 is free software: you can redistribute it and/or modify it
+   under the terms of either:
+
+     * the GNU Lesser General Public License as published by the Free
+       Software Foundation; either version 3 of the License, or (at
+       your option) any later version.
+
+   or
+
+     * the GNU General Public License as published by the Free
+       Software Foundation; either version 2 of the License, or (at
+       your option) any later version.
+
+   or both in parallel, as here.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received copies of the GNU General Public License and
+   the GNU Lesser General Public License along with this program.  If
+   not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <config.h>
@@ -96,7 +107,7 @@ _idn2_label_test (int what, const uint32_t * label, size_t llen)
 	    return IDN2_MALLOC;
 	  return IDN2_NFC;
 	}
-      ok = llen == plen && memcmp (label, p, plen) == 0;
+      ok = llen == plen && memcmp (label, p, plen * sizeof (*label)) == 0;
       free (p);
       if (!ok)
 	return IDN2_NOT_NFC;
