@@ -1,5 +1,5 @@
 /* idn2.c - command line interface to libidn2
-   Copyright (C) 2011 Simon Josefsson
+   Copyright (C) 2011-2016 Simon Josefsson
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -34,7 +34,6 @@
 #include "error.h"
 #include "gettext.h"
 #define _(String) dgettext (PACKAGE, String)
-#define N_(String) gettext_noop (String)
 #include "progname.h"
 #include "version-etc.h"
 #include "localcharset.h"
@@ -44,7 +43,7 @@
 #include "blurbs.h"
 
 #define GREETING							\
-  "Copyright (C) 2011  Simon Josefsson\n"				\
+  "Copyright (C) 2011-2013  Simon Josefsson\n"				\
   "This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.\n" \
   "This is free software, and you are welcome to redistribute it\n" \
   "under certain conditions; type `show c' for details.\n\n"
@@ -97,7 +96,7 @@ Mandatory arguments to long options are mandatory for short options too.\n\
   exit (status);
 }
 
-void
+static void
 hexdump (const char *prefix, const char *str)
 {
   uint8_t *u8;
@@ -107,14 +106,14 @@ hexdump (const char *prefix, const char *str)
 
   u8 = u8_strconv_from_locale (str);
   if (u8)
-    u32 = u8_to_u32 (u8, strlen (u8), NULL, &u32len);
+    u32 = u8_to_u32 (u8, strlen ((char *) u8), NULL, &u32len);
 
   for (i = 0; i < strlen (str); i++)
     fprintf (stderr, "%s[%lu] = 0x%02x\n",
 	     prefix, (unsigned long) i, str[i] & 0xFF);
 
-  if (u8 && strcmp (str, u8) != 0)
-    for (i = 0; i < strlen (u8); i++)
+  if (u8 && strcmp (str, (char *) u8) != 0)
+    for (i = 0; i < strlen ((char *) u8); i++)
       fprintf (stderr, "UTF-8 %s[%lu] = 0x%02x\n",
 	       prefix, (unsigned long) i, u8[i] & 0xFF);
 
@@ -158,8 +157,8 @@ main (int argc, char *argv[])
 
   if (!args_info.quiet_given
       && args_info.inputs_num == 0 && isatty (fileno (stdin)))
-    fprintf (stderr, _("Type each input string on a line by itself, "
-		       "terminated by a newline character.\n"));
+    fprintf (stderr, "%s", _("Type each input string on a line by itself, "
+			     "terminated by a newline character.\n"));
 
   do
     {
@@ -176,10 +175,10 @@ main (int argc, char *argv[])
 	  if (feof (stdin))
 	    break;
 
-	  error (EXIT_FAILURE, errno, _("input error"));
+	  error (EXIT_FAILURE, errno, "%s", _("input error"));
 	}
 
-      if (readbuf[strlen (readbuf) - 1] == '\n')
+      if (strlen(readbuf) > 0 && readbuf[strlen (readbuf) - 1] == '\n' )
 	readbuf[strlen (readbuf) - 1] = '\0';
 
       if (strcmp (readbuf, "show w") == 0)
