@@ -1,5 +1,5 @@
 /* test-lookup.c --- Self tests for IDNA processing
-   Copyright (C) 2011 Simon Josefsson
+   Copyright (C) 2011-2016 Simon Josefsson
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -548,7 +548,7 @@ static const struct idna idna[] = {
    /* Don't resolve as xn--doublediac2udn-cmb526fod.example */
   },
   {"\x6e\x65\x77\x6e\x6f\x72\x6d\xf0\xaf\xa1\xb4\xc3\xad\x64\x6e\x2e\x65\x78\x61\x6d\x70\x6c\x65",
-   "xn--newnormdn-m5a7856x.example", IDN2_DISALLOWED
+   "xn--newnormdn-m5a7856x.example", IDN2_NOT_NFC
    /* 4-2-2-1 newnorm<2F874><00ED>dn.example -> newnorm<5F33><00ED>dn.example should not become <5F53> */
    /* Don't resolve as xn--newnormdn-m5a9396x.example */
    /* Don't resolve as xn--newnormdn-m5a9968x.example */
@@ -690,6 +690,11 @@ static const struct idna idna[] = {
     /* Contextj: U+750 U+0300 U+200C U+0300 U+0300 U+0300 U+0750.
        U+0300 is T, U+0750 is D. */
   },
+  {
+    "\xd8\xa8\xe2\x80\x8c\xd8\xa7\xe2\x80\x8c\xd8\xa7", "", IDN2_CONTEXTJ
+    /* Contextj: U+0628 U+200C U+0627 U+200C U+0627.
+       http://article.gmane.org/gmane.ietf.idnabis/7366 */
+  },
   {
     "\xc2\xb7", "xn--uba", IDN2_OK
     /* Contexto: Lookup of U+00B7 should succeed. */
